#ifndef DEFINES_H
#define DEFINES_H

#define VERSION "LibreMediaServer-Audio 0.1.3"
#define COPYRIGHT "(C) 2014-2024 Santi Norena lms@criptomart.net"
#define LICENSE "GPL 3 License. See LICENSE.txt and credits.txt for details"

#define LAYERS_NUMBER 4 // esto tiene que desaparecer

#define DEFAULT_FILE "lms-audio.xlm"

const int DurationSeconds = 1;
const int ToneSampleRateHz = 600;
const int DataSampleRateHz = 44100;
const int BufferSize      = 262144;

#define SUSPEND_LABEL       "Pause playback"
#define RESUME_LABEL        "Resume playback"

#define PLAY_LABEL          "Playing"
#define STOP_LABEL          "Stopped"
#define PAUSE_LABEL         "Pause"
#define IDDLE_LABEL         "Iddle playback"

#define VOLUME_LABEL        "Volume"
#define PROGRESS_LABEL      "Progress"
#define PROGRESS_TIME_LABEL "Current"
#define REMAINING_TIME      "Remaining Time: "
#define TOTAL_TIME_LABEL    "Total"
#define FILE_LABEL          "File:"
#define FOLDER_LABEL        "Folder:"
#define STATUS_LABEL        "Status: "

#define NOTIFY_INTERVAL 150
#define PULL_TIMER_INTERVAL 10

// struct where save the DMX settings for each layer
struct dmxSetting {
    int address;
    uint universe;
    bool updated;
    int layer;
};

// Media Information for MELIn packages. v1.0
struct MediaFile {
    quint8 Number; // 0-based contiguous index of the media.
    QString MediaName;// Media name.
    quint32 MediaLength;// Media length (in frames).
};

// Media Library for ELin packages v1.0
struct MediaFolder {
    quint8 m_Id; // Library id.
    QString m_Name;// Library name.
    quint8 m_ElementCount;// Number of elements in the library.
    QList<MediaFile> m_MediaInformation; // Pointer to the Medias Information List of this Library
};

enum Status
{
    Stopped,
    Paused,
    PlayingOnce,
    PlayingLoop,
};

#endif // DEFINES_H

